/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include "Jet.h"
#include "RGB.h"

Jet::Jet(int id, int x, int y, Map *imageMap, int side, int color, int dots) :
	imageStart(0,0)
{
	this->id = id;
	this->x = x;
	this->y = y;
	this->ystart = 0;
	this->imageMap = imageMap;
	this->side = side;
	this->color = color;
	this->dots = dots;
}

Coord *Jet::getPreferredImageStart(Coord *start)
{
	start->x = this->x;
	start->y = this->y;
	
	return start;
}

int Jet::getPreferredEnd()
{
	// image ending point minus our offset from printer head
	return imageStart.x + imageMap->width - x;
}


void Jet::setImageStart(Coord *imageStart)
{
    this->imageStart.x = imageStart->x;
    this->imageStart.y = imageStart->y;
}

bool Jet::finished()
{
     return this->ystart > this->imageMap->height;
}

bool Jet::needsCover(int y)
{
	if(!insideImage(imageStart.x,y+this->y))
	{
		return false;
	}
	
	int iy = y + this->y - this->imageStart.y; // y from image

	if(iy < ystart)
		return false;

	for(int ix = 0; ix < imageMap->width; ix++)
	{
		if(needsPainting(ix,iy,0))
		{
			return true;
		}
	}

	return false;
}

bool Jet::insideImage(int x, int y)
{
	if( x < imageStart.x || x >= imageStart.x + imageMap->width || y < imageStart.y || y >= imageStart.y + imageMap->height )
		return false;

	return true;
}

/* finds the startX and endX for the head so the jet can cover the image */
bool Jet::findStartEnd(int y, int *startX, int *endX)
{
	int iy = y + this->y - this->imageStart.y; // y from image

	int d = ((this->side == LEFT_JET) ? 1 : 0);

	int minImageStartX = imageMap->width, maxImageEndX = -1;

	while(d < dots * 2)
	{
		int ix;
		
		for(ix = 0; ix < minImageStartX; ix ++)
		{
			if(needsPainting(ix,iy+d,0))
			{
				break;
			}
		}
		minImageStartX = ix;
		
		for(ix = imageMap->width; ix > maxImageEndX; ix --)
		{
			if(needsPainting(ix,iy+d,0))
			{
				break;
			}
		}
		maxImageEndX = ix;

		d+=2;
	}

	if(maxImageEndX == -1)
		return false;

	*startX = minImageStartX + imageStart.x - x;
	*endX = maxImageEndX + imageStart.x - x;

	return true;
}


inline bool Jet::needsPainting(int ix, int iy, int bit)
{
	if(ix <0 || ix >= imageMap->width || iy <0 || iy >= imageMap->height)
		return false;

	RGB *rgb;
	rgb = (RGB *)getMapCell(imageMap, ix, iy);
	
	unsigned char brightness = color == 0 ? rgb->r : color == 1 ? rgb->g : rgb->b;

	if(brightness)
		return true;

	return false;
}

void Jet::paint(int startX, int endX, int y, LexmarkProtocol *protocol)
{
	int jetStartX,jetEndX, jetY;

    jetStartX = startX + this->x;
    jetY = y + this->y;

    jetEndX = endX + this->x;

    for(int jetX = jetStartX; jetX < jetEndX; jetX++)
    {
		for(int j = 0; j < dots; j++)
		{
			//if we are the left jet, we paint the odd dots. The right jet paints the even dots.
			int nozzle = ((this->side == LEFT_JET) ? 1 : 0) + j * 2;

			int y = jetY + nozzle;

			int ix = jetX-this->imageStart.x;  //x from image coordinates
			int iy = y-this->imageStart.y;  //y from image coordinates

			//are we in the image? fixme, cut out edges beforehand
			if(ix >= 0 && iy >=0 && ix < this->imageMap->width &&
			   iy < this->imageMap->height)
			{
				if(needsPainting(ix,iy, nozzle))
				{
					protocol->paint(jetStartX,jetX,nozzle, this->color);
				}
			}
		}
    }
	
    this->ystart = jetY - this->imageStart.y + dots * 2;
}

void Jet::getImageRange(Rect *dest)
{
	dest->x = imageStart.x - x;
	dest->y = imageStart.y - y;
	dest->width = imageMap->width;
	dest->height = imageMap->height;
}

void Jet::getRange(Rect *dest)
{
	dest->x = x;
	dest->y = y;
	dest->width = 1;
	dest->height = dots*2;
}

