/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include "errors.h"
#include "PPMRaw.h"
#include "RGB.h"

#define BYTES_PER_PIXEL 3

PPMRaw::PPMRaw(Reader *reader,bool readMagic)
{
	this->reader = reader;
	this->start = 0;
	this->end = 0;
	this->width = -1;
	this->height = -1;
	this->bytes = NULL;
	this->maxval = 0;
	
	
	readHeader(readMagic);
}

void PPMRaw::getRGB(RGB *rgb, int x, int y)
{
	int pos = (x + y * this->width) * BYTES_PER_PIXEL - this->start;

    if(pos < 0 || pos > this->end)
    {
		die("Illegal RGB position, not cached (%d,%d)",x,y);
    }

	rgb->r = bytes[pos];
	rgb->g = bytes[pos+1];
	rgb->b = bytes[pos+2];
	return;
}

void PPMRaw::readHeader(bool readMagic)
{
	readHeader(&width,&height,&maxval, reader, readMagic);

	this->byte_width = this->width * BYTES_PER_PIXEL;
}



void PPMRaw::readHeader(int *width, int *height, int *maxval, Reader *reader, bool readMagic)
{
    //read magic number
	char line[256];
	
	if(readMagic)
	{
		reader->readLine(line,255);
		
		if(!PPMRaw::isMagic(line))
		{
			die("Bad magic number in image file, got %s", line);
		}
	}

	int order = 0;
	
    //get width and height
    while(reader->readLine(line,255) != -1)
	{
		if(line[0] == '#')
		{
			continue;
		}
		
		if ( order == 0 && sscanf(line,"%d %d",width, height) == 2)
		{
			order = 1;
		}
		else if ( order == 1 && sscanf(line,"%d", maxval) == 1)
		{
			return;
		}
		else
		{
			die("Not understood line in image file, %s", line);
		}
    }
		
	die("Got eof looking for width and height or maxval");
}


static void test(char *filename)
{
	int old_j, j,x,y;
    FILE *fd;
	Image *image;

	fd = fopen(filename,"r");

	if(fd == NULL)
		die("Couldn't open %s for reading",filename);

	Reader r(fd,true );

	PPMRaw PPMRaw(&r,true);

	image = (Image *)&PPMRaw;
    
	old_j=0;

    for(j = 1; j <= image->height; j += 1)
    {
		image->cacheLines(old_j,j+1);

		for(y = old_j; y < j;y ++)
		{
			for(x = 0; x < image->width; x++)
			{
				RGB rgb;

				image->getRGB(&rgb,x,y);
				
				if(rgb.r + rgb.g + rgb.b > 0.5)
				{
					putc('X',stdout);
				}
				else
				{
					putc(' ',stdout);
				}
			}
			old_j = j;
			putc('\n',stdout);
		}
	}
	
}


bool PPMRaw::isMagic(char *line)
{
	if(strcmp(line,"P6") == 0)
		return true;
	
	return false;
}


#ifdef PPM_TEST

int main(int argc, char **argv)
{
	if(argc != 2)
	{
		die("Usage: %s <ppm file>\n",argv[0]);
	}
	
	test(argv[1]);
	
}

#endif
